## Sid Meier's Civilization 4
## Copyright Firaxis Games 2006
## 
## CvEventManager
## This class is passed an argsList from CvAppInterface.onEvent
## The argsList can contain anything from mouse location to key info
## The EVENTLIST that are being notified can be found 


from CvPythonExtensions import *
import CvUtil
import CvScreensInterface
import CvDebugTools
import CvWBPopups
import PyHelpers
import Popup as PyPopup
import CvCameraControls
import CvTopCivs
import sys
import CvWorldBuilderScreen
import CvAdvisorUtils
import CvTechChooser
# Rise of Mankind start
import GodsOfOld
# import pickle
import cPickle as pickle
import CvGameUtils
import BugOptions
import BugCore
import BugUtil
import OOSLogger
RevDCMOpt = BugCore.game.RevDCM
# Rise of Mankind end

gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
PyCity = PyHelpers.PyCity
PyGame = PyHelpers.PyGame

# globals
###################################################
class CvEventManager:
	def __init__(self):
		#################### ON EVENT MAP ######################
		#print "EVENTMANAGER INIT"
				
		self.bCtrl = False
		self.bShift = False
		self.bAlt = False
		self.bAllowCheats = False
		
		# OnEvent Enums
		self.EventLButtonDown=1
		self.EventLcButtonDblClick=2
		self.EventRButtonDown=3
		self.EventBack=4
		self.EventForward=5
		self.EventKeyDown=6
		self.EventKeyUp=7
	
		self.__LOG_MOVEMENT = 0
		self.__LOG_BUILDING = 0
		self.__LOG_COMBAT = 0
		self.__LOG_CONTACT = 0
		self.__LOG_IMPROVEMENT =0
		self.__LOG_CITYLOST = 0
		self.__LOG_CITYBUILDING = 0
		self.__LOG_TECH = 0
		self.__LOG_UNITBUILD = 0
		self.__LOG_UNITKILLED = 1
		self.__LOG_UNITLOST = 0
		self.__LOG_UNITPROMOTED = 0
		self.__LOG_UNITSELECTED = 0
		self.__LOG_UNITPILLAGE = 0
		self.__LOG_GOODYRECEIVED = 0
		self.__LOG_GREATPERSON = 0
		self.__LOG_RELIGION = 0
		self.__LOG_RELIGIONSPREAD = 0
		self.__LOG_GOLDENAGE = 0
		self.__LOG_ENDGOLDENAGE = 0
		self.__LOG_WARPEACE = 0
		self.__LOG_PUSH_MISSION = 0
		
		# RoM start Next War tracks cities that have been razed
		self.iArcologyCityID = -1
		
		#Net Messages
		self.m_iNetMessage_Inquisitor = 0
		
# Rise of Mankind 2.7
		# def onBuildingBuilt / Initializing of building upgrade line
		self.asBuildingUpgradeLine_Upgrade_Name = [
		["BUILDING_KNIGHT_STABLE"],
		["BUILDING_ARCOLOGY_SHIELDING"],
		["BUILDING_ADVANCED_SHIELDING"],
		["BUILDING_HARBOR","BUILDING_CARTHAGE_COTHON"],
		["BUILDING_PORT"],
		["BUILDING_COMMERCIAL_PORT"],
		["BUILDING_INTERNATIONAL_PORT"],
		["BUILDING_COMMERCIAL_AIRPORT"],
		["BUILDING_HIGH_WALLS"],
		["BUILDING_CUSTOM_HOUSE","BUILDING_PORTUGAL_FEITORIA"],
		["BUILDING_DRYDOCK"],
		["BUILDING_NAVAL_YARD"],
		["BUILDING_STEEL_MILL"],
		["BUILDING_FOOD_PROCESSING_PLANT"],
		["BUILDING_ARCOLOGY"],
		["BUILDING_MODERN_GRANARY"],
		["BUILDING_GARRISON"],
		["BUILDING_NUCLEAR_PLANT","BUILDING_HYDRO_PLANT","BUILDING_FUSION_PLANT"]]
		self.asBuildingUpgradeLine_Obsolete_Name = [
		["BUILDING_STABLE","BUILDING_MONGOL_GER"],
		["BUILDING_ANTI_MISSILE_BATTERIES","BUILDING_ARCOLOGY"],
		["BUILDING_ANTI_MISSILE_BATTERIES","BUILDING_ARCOLOGY","BUILDING_ARCOLOGY_SHIELDING"],
		["BUILDING_RIVER_PORT"],
		["BUILDING_RIVER_PORT","BUILDING_HARBOR","BUILDING_CARTHAGE_COTHON"],
		["BUILDING_RIVER_PORT","BUILDING_HARBOR","BUILDING_CARTHAGE_COTHON","BUILDING_PORT"],
		["BUILDING_RIVER_PORT","BUILDING_HARBOR","BUILDING_CARTHAGE_COTHON","BUILDING_PORT","BUILDING_COMMERCIAL_PORT"],
		["BUILDING_AIRPORT"],
		["BUILDING_WALLS","BUILDING_CELTIC_DUN"],
		["BUILDING_TOLL_HOUSE"],
		["BUILDING_SHIPYARD"],
		["BUILDING_SHIPYARD","BUILDING_DRYDOCK"],
		["BUILDING_FORGE","BUILDING_MALI_MINT","BUILDING_IRON_FORGE"],
		["BUILDING_BAKERY"],
		["BUILDING_ANTI_MISSILE_BATTERIES"],
		["BUILDING_GRANARY","BUILDING_INCAN_TERRACE"],
		["BUILDING_BARRACKS","BUILDING_ZULU_IKHANDA","BUILDING_ASSYRIA_EKAL"],
		["BUILDING_COAL_PLANT","BUILDING_JAPANESE_SHALE_PLANT","BUILDING_OIL_PLANT"]]
		self.aiBuildingUpgradeLine_Upgrade_InfoTypeNum = []
		self.aiBuildingUpgradeLine_Obsolete_InfoTypeNum = []
		
		self.iBUILDING_CRUSADE = ""
# Rise of Mankind 2.7

		## EVENTLIST
		self.EventHandlerMap = {
			'mouseEvent'			: self.onMouseEvent,
			'kbdEvent' 				: self.onKbdEvent,
			'ModNetMessage'					: self.onModNetMessage,
			'Init'					: self.onInit,
			'Update'				: self.onUpdate,
			'UnInit'				: self.onUnInit,
			'OnSave'				: self.onSaveGame,
			'OnPreSave'				: self.onPreSave,
			'OnLoad'				: self.onLoadGame,
			'GameStart'				: self.onGameStart,
			'GameEnd'				: self.onGameEnd,
			'plotRevealed' 			: self.onPlotRevealed,
			'plotFeatureRemoved' 	: self.onPlotFeatureRemoved,
			'plotPicked'			: self.onPlotPicked,
			'nukeExplosion'			: self.onNukeExplosion,
			'gotoPlotSet'			: self.onGotoPlotSet,
			'BeginGameTurn'			: self.onBeginGameTurn,
			'EndGameTurn'			: self.onEndGameTurn,
			'BeginPlayerTurn'		: self.onBeginPlayerTurn,
			'EndPlayerTurn'			: self.onEndPlayerTurn,
			'endTurnReady'			: self.onEndTurnReady,
			'combatResult' 			: self.onCombatResult,
		  'combatLogCalc'	 		: self.onCombatLogCalc,
		  'combatLogHit'				: self.onCombatLogHit,
			'improvementBuilt' 		: self.onImprovementBuilt,
			'improvementDestroyed' 		: self.onImprovementDestroyed,
			'routeBuilt' 		: self.onRouteBuilt,
			'firstContact' 			: self.onFirstContact,
			'cityBuilt' 			: self.onCityBuilt,
			'cityRazed'				: self.onCityRazed,
			'cityAcquired' 			: self.onCityAcquired,
			'cityAcquiredAndKept' 	: self.onCityAcquiredAndKept,
			'cityLost'				: self.onCityLost,
			'cultureExpansion' 		: self.onCultureExpansion,
			'cityGrowth' 			: self.onCityGrowth,
			'cityDoTurn' 			: self.onCityDoTurn,
			'cityBuildingUnit'	: self.onCityBuildingUnit,
			'cityBuildingBuilding'	: self.onCityBuildingBuilding,
			'cityRename'				: self.onCityRename,
			'cityHurry'				: self.onCityHurry,
			'selectionGroupPushMission'		: self.onSelectionGroupPushMission,
			'unitMove' 				: self.onUnitMove,
			'unitSetXY' 			: self.onUnitSetXY,
			'unitCreated' 			: self.onUnitCreated,
			'unitBuilt' 			: self.onUnitBuilt,
			'unitKilled'			: self.onUnitKilled,
			'unitLost'				: self.onUnitLost,
			'unitPromoted'			: self.onUnitPromoted,
			'unitSelected'			: self.onUnitSelected, 
			'UnitRename'				: self.onUnitRename,
			'unitPillage'				: self.onUnitPillage,
			'unitSpreadReligionAttempt'	: self.onUnitSpreadReligionAttempt,
			'unitGifted'				: self.onUnitGifted,
			'unitBuildImprovement'				: self.onUnitBuildImprovement,
			'goodyReceived'        	: self.onGoodyReceived,
			'greatPersonBorn'      	: self.onGreatPersonBorn,
			'buildingBuilt' 		: self.onBuildingBuilt,
			'projectBuilt' 			: self.onProjectBuilt,
			'techAcquired'			: self.onTechAcquired,
			'techSelected'			: self.onTechSelected,
			'religionFounded'		: self.onReligionFounded,
			'religionSpread'		: self.onReligionSpread, 
			'religionRemove'		: self.onReligionRemove, 
			'corporationFounded'	: self.onCorporationFounded,
			'corporationSpread'		: self.onCorporationSpread, 
			'corporationRemove'		: self.onCorporationRemove, 
			'goldenAge'				: self.onGoldenAge,
			'endGoldenAge'			: self.onEndGoldenAge,
			'chat' 					: self.onChat,
			'victory'				: self.onVictory,
			'vassalState'			: self.onVassalState,
			'changeWar'				: self.onChangeWar,
			'setPlayerAlive'		: self.onSetPlayerAlive,
			'playerChangeStateReligion'		: self.onPlayerChangeStateReligion,
			'playerGoldTrade'		: self.onPlayerGoldTrade,
			'windowActivation'		: self.onWindowActivation,
			'gameUpdate'			: self.onGameUpdate,		# sample generic event
		}

		################## Events List ###############################
		#
		# Dictionary of Events, indexed by EventID (also used at popup context id)
		#   entries have name, beginFunction, applyFunction [, randomization weight...]
		#
		# Normal events first, random events after
		#	
		################## Events List ###############################
		self.Events={
			CvUtil.EventEditCityName : ('EditCityName', self.__eventEditCityNameApply, self.__eventEditCityNameBegin),
			CvUtil.EventEditCity : ('EditCity', self.__eventEditCityApply, self.__eventEditCityBegin),
			CvUtil.EventPlaceObject : ('PlaceObject', self.__eventPlaceObjectApply, self.__eventPlaceObjectBegin),
			CvUtil.EventAwardTechsAndGold: ('AwardTechsAndGold', self.__eventAwardTechsAndGoldApply, self.__eventAwardTechsAndGoldBegin),
			CvUtil.EventEditUnitName : ('EditUnitName', self.__eventEditUnitNameApply, self.__eventEditUnitNameBegin),
			CvUtil.EventWBAllPlotsPopup : ('WBAllPlotsPopup', self.__eventWBAllPlotsPopupApply, self.__eventWBAllPlotsPopupBegin),
			CvUtil.EventWBLandmarkPopup : ('WBLandmarkPopup', self.__eventWBLandmarkPopupApply, self.__eventWBLandmarkPopupBegin),
			CvUtil.EventWBScriptPopup : ('WBScriptPopup', self.__eventWBScriptPopupApply, self.__eventWBScriptPopupBegin),
			CvUtil.EventWBStartYearPopup : ('WBStartYearPopup', self.__eventWBStartYearPopupApply, self.__eventWBStartYearPopupBegin),
			CvUtil.EventShowWonder: ('ShowWonder', self.__eventShowWonderApply, self.__eventShowWonderBegin),
		}

#################### EVENT STARTERS ######################
	def handleEvent(self, argsList):
		'EventMgr entry point'
		# extract the last 6 args in the list, the first arg has already been consumed
		self.origArgsList = argsList	# point to original
		tag = argsList[0]				# event type string
		idx = len(argsList)-6
		bDummy = false
		self.bDbg, bDummy, self.bAlt, self.bCtrl, self.bShift, self.bAllowCheats = argsList[idx:]
		ret = 0
		if self.EventHandlerMap.has_key(tag):
			fxn = self.EventHandlerMap[tag]
			ret = fxn(argsList[1:idx])
		return ret
		
#################### EVENT APPLY ######################	
	def beginEvent( self, context, argsList=-1 ):
		'Begin Event'
		entry = self.Events[context]
		return entry[2]( argsList )
	
	def applyEvent( self, argsList ):
		'Apply the effects of an event '
		context, playerID, netUserData, popupReturn = argsList
		
		if context == CvUtil.PopupTypeEffectViewer:
			return CvDebugTools.g_CvDebugTools.applyEffectViewer( playerID, netUserData, popupReturn )
		
		entry = self.Events[context]
				
		if ( context not in CvUtil.SilentEvents ):
			self.reportEvent(entry, context, (playerID, netUserData, popupReturn) )
		return entry[1]( playerID, netUserData, popupReturn )   # the apply function

	def reportEvent(self, entry, context, argsList):
		'Report an Event to Events.log '
		if (gc.getGame().getActivePlayer() != -1):
			message = "DEBUG Event: %s (%s)" %(entry[0], gc.getActivePlayer().getName())
			CyInterface().addImmediateMessage(message,"")
			CvUtil.pyPrint(message)
		return 0
		
#################### ON EVENTS ######################
	def onKbdEvent(self, argsList):
		'keypress handler - return 1 if the event was consumed'

		eventType,key,mx,my,px,py = argsList
		game = gc.getGame()
		
		if (self.bAllowCheats):
			# notify debug tools of input to allow it to override the control
			argsList = (eventType,key,self.bCtrl,self.bShift,self.bAlt,mx,my,px,py,gc.getGame().isNetworkMultiPlayer())
			if ( CvDebugTools.g_CvDebugTools.notifyInput(argsList) ):
				return 0
		
		if ( eventType == self.EventKeyDown ):
			theKey=int(key)
			
			CvCameraControls.g_CameraControls.handleInput( theKey )

			if (self.bAllowCheats):
				# Shift - T (Debug - No MP)
				if (theKey == int(InputTypes.KB_T)):
					if ( self.bShift ):
						self.beginEvent(CvUtil.EventAwardTechsAndGold)
						#self.beginEvent(CvUtil.EventCameraControlPopup)
						return 1
							
				elif (theKey == int(InputTypes.KB_W)):
					if ( self.bShift and self.bCtrl):
						self.beginEvent(CvUtil.EventShowWonder)
						return 1
							
				# Shift - ] (Debug - currently mouse-overd unit, health += 10
				elif (theKey == int(InputTypes.KB_LBRACKET) and self.bShift ):
					unit = CyMap().plot(px, py).getUnit(0)
					if ( not unit.isNone() ):
						d = min( unit.maxHitPoints()-1, unit.getDamage() + 10 )
						unit.setDamage( d, PlayerTypes.NO_PLAYER )
					
				# Shift - [ (Debug - currently mouse-overd unit, health -= 10
				elif (theKey == int(InputTypes.KB_RBRACKET) and self.bShift ):
					unit = CyMap().plot(px, py).getUnit(0)
					if ( not unit.isNone() ):
						d = max( 0, unit.getDamage() - 10 )
						unit.setDamage( d, PlayerTypes.NO_PLAYER )
					
				elif (theKey == int(InputTypes.KB_F1)):
					if ( self.bShift ):
						CvScreensInterface.replayScreen.showScreen(False)
						return 1
					# don't return 1 unless you want the input consumed
				
				elif (theKey == int(InputTypes.KB_F2)):
					if ( self.bShift ):
						import CvDebugInfoScreen
						CvScreensInterface.showDebugInfoScreen()
						return 1
				
				elif (theKey == int(InputTypes.KB_F3)):
					if ( self.bShift ):
						CvScreensInterface.showDanQuayleScreen(())
						return 1
						
				elif (theKey == int(InputTypes.KB_F4)):
					if ( self.bShift ):
						CvScreensInterface.showUnVictoryScreen(())
						return 1
											
		return 0

	def onModNetMessage(self, argsList):
		'Called whenever CyMessageControl().sendModNetMessage() is called - this is all for you modders!'
		
		iData1, iData2, iData3, iData4, iData5 = argsList
		
		print("Modder's net message!")
		
		CvUtil.pyPrint( 'onModNetMessage' )
		
		# Rise of Mankind start
		iMessageID = iData1
		
		#Inquisitor's Effect on a City
		if ( iMessageID == self.m_iNetMessage_Inquisitor ):
			
			iPlotX = iData2
			iPlotY = iData3
			iOwner = iData4
			iUnitID = iData5
			
			pPlot = CyMap( ).plot( iPlotX, iPlotY )
			pCity = pPlot.getPlotCity( )
			pPlayer = gc.getPlayer( iOwner )
			pUnit = pPlayer.getUnit( iUnitID )
			
			self.doInquisitorPersecution( pCity, pUnit )
# Rise of Mankind 2.63
		# NetModMessage 410
		# SettlersEventManager.py / Colonist
		# Add additional buildings and change city size
		if ( iMessageID == 410 ):
			pPlayer = gc.getPlayer(iData3)
			pCity = pPlayer.getCity(iData2)
			pCity.setPopulation(3)
			self.addCityBuildings(pCity, "BUILDINGCLASS_BARRACKS")
			self.addCityBuildings(pCity, "BUILDINGCLASS_GRANARY")
			self.addCityBuildings(pCity, "BUILDINGCLASS_FORGE")
			self.addCityBuildings(pCity, "BUILDINGCLASS_MARKET")
			if pCity.plot().isCoastalLand():
				self.addCityBuildings(pCity, "BUILDINGCLASS_HARBOR")
				self.addCityBuildings(pCity, "BUILDINGCLASS_LIGHTHOUSE")
				self.addCityBuildings(pCity, "BUILDINGCLASS_FISHERMAN_HUT")
			
		# NetModMessage 410
		# SettlersEventManager.py / Pioneer
		# Add additional buildings and change city size
		if ( iMessageID == 411 ):
			pPlayer = gc.getPlayer(iData3)
			pCity = pPlayer.getCity(iData2)
			pCity.setPopulation(4)
			self.addCityBuildings(pCity, "BUILDINGCLASS_GARRISON")
			self.addCityBuildings(pCity, "BUILDINGCLASS_GRANARY")
			self.addCityBuildings(pCity, "BUILDINGCLASS_FORGE")
			self.addCityBuildings(pCity, "BUILDINGCLASS_COURTHOUSE")
			self.addCityBuildings(pCity, "BUILDINGCLASS_MARKET")
			self.addCityBuildings(pCity, "BUILDINGCLASS_STABLE")
			self.addCityBuildings(pCity, "BUILDINGCLASS_GROCER")
			self.addCityBuildings(pCity, "BUILDINGCLASS_DOCTOR")
			self.addCityBuildings(pCity, "BUILDINGCLASS_BANK")
			self.addCityBuildings(pCity, "BUILDINGCLASS_LIBRARY")
			if pCity.plot().isCoastalLand():
				self.addCityBuildings(pCity, "BUILDINGCLASS_PORT")
				self.addCityBuildings(pCity, "BUILDINGCLASS_LIGHTHOUSE")
				self.addCityBuildings(pCity, "BUILDINGCLASS_FISHERMAN_HUT")
						
	def addCityBuildings(self, pCity, szBuilding):
		iBuilding = gc.getInfoTypeForString(szBuilding)
		iUniqueBuilding = gc.getCivilizationInfo(gc.getPlayer(pCity.getOwner()).getCivilizationType()).getCivilizationBuildings(iBuilding)
		if iUniqueBuilding > -1:
			if pCity.canConstruct(iUniqueBuilding, False, True, False):
				pCity.setNumRealBuilding(iUniqueBuilding, 1)
# Rise of Mankind 2.63
				
	def onInit(self, argsList):
		'Called when Civ starts up'
		CvUtil.pyPrint( 'OnInit' )
		
	def onUpdate(self, argsList):
		'Called every frame'
		fDeltaTime = argsList[0]
		
		# allow camera to be updated
		CvCameraControls.g_CameraControls.onUpdate( fDeltaTime )
		
	def onWindowActivation(self, argsList):
		'Called when the game window activates or deactivates'
		bActive = argsList[0]
		
	def onUnInit(self, argsList):
		'Called when Civ shuts down'
		CvUtil.pyPrint('OnUnInit')
	
	def onPreSave(self, argsList):
		"called before a game is actually saved"
		CvUtil.pyPrint('OnPreSave')
	
	def onSaveGame(self, argsList):
		"return the string to be saved - Must be a string"
		return ""

	def onLoadGame(self, argsList):
		CvAdvisorUtils.resetNoLiberateCities()
# Rise of Mankind 2.7
		for iBuildingUpgradeLine_OuterItem in range(self.asBuildingUpgradeLine_Upgrade_Name.__len__()):
			for iBuildingUpgradeLine_InnerItem_Upgrade in range(self.asBuildingUpgradeLine_Upgrade_Name[iBuildingUpgradeLine_OuterItem].__len__()):
				for iBuildingUpgradeLine_InnerItem_Obsolete in range(self.asBuildingUpgradeLine_Obsolete_Name[iBuildingUpgradeLine_OuterItem].__len__()):
					self.aiBuildingUpgradeLine_Upgrade_InfoTypeNum.append(CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), self.asBuildingUpgradeLine_Upgrade_Name[iBuildingUpgradeLine_OuterItem][iBuildingUpgradeLine_InnerItem_Upgrade]))
					self.aiBuildingUpgradeLine_Obsolete_InfoTypeNum.append(CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), self.asBuildingUpgradeLine_Obsolete_Name[iBuildingUpgradeLine_OuterItem][iBuildingUpgradeLine_InnerItem_Obsolete]))

		self.iBUILDING_CRUSADE = gc.getInfoTypeForString("BUILDING_CRUSADE")
# Rise of Mankind 2.7
		return 0

	def onGameStart(self, argsList):
		'Called at the start of the game'
		if (gc.getGame().getGameTurnYear() == gc.getDefineINT("START_YEAR") and not gc.getGame().isOption(GameOptionTypes.GAMEOPTION_ADVANCED_START)):
			for iPlayer in range(gc.getMAX_PLAYERS()):
				player = gc.getPlayer(iPlayer)
				if (player.isAlive() and player.isHuman()):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
					popupInfo.setText(u"showDawnOfMan")
					popupInfo.addPopup(iPlayer)
		else:
			CyInterface().setSoundSelectionReady(true)

		if gc.getGame().isPbem():
			for iPlayer in range(gc.getMAX_PLAYERS()):
				player = gc.getPlayer(iPlayer)
				if (player.isAlive() and player.isHuman()):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_DETAILS)
					popupInfo.setOption1(true)
					popupInfo.addPopup(iPlayer)

		CvAdvisorUtils.resetNoLiberateCities()
# Rise of Mankind 2.63 - init building upgrade lines, can not be done earlier as the game inits python before it inits xml...
		for iBuildingUpgradeLine_OuterItem in range(self.asBuildingUpgradeLine_Upgrade_Name.__len__()):
			for iBuildingUpgradeLine_InnerItem_Upgrade in range(self.asBuildingUpgradeLine_Upgrade_Name[iBuildingUpgradeLine_OuterItem].__len__()):
				for iBuildingUpgradeLine_InnerItem_Obsolete in range(self.asBuildingUpgradeLine_Obsolete_Name[iBuildingUpgradeLine_OuterItem].__len__()):
					self.aiBuildingUpgradeLine_Upgrade_InfoTypeNum.append(CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), self.asBuildingUpgradeLine_Upgrade_Name[iBuildingUpgradeLine_OuterItem][iBuildingUpgradeLine_InnerItem_Upgrade]))
					self.aiBuildingUpgradeLine_Obsolete_InfoTypeNum.append(CvUtil.findInfoTypeNum(gc.getBuildingInfo, gc.getNumBuildingInfos(), self.asBuildingUpgradeLine_Obsolete_Name[iBuildingUpgradeLine_OuterItem][iBuildingUpgradeLine_InnerItem_Obsolete]))
		# def onBuildingBuilt / Additional tests variable
		self.iBUILDING_CRUSADE = gc.getInfoTypeForString("BUILDING_CRUSADE")
# Rise of Mankind

	def onGameEnd(self, argsList):
		'Called at the End of the game'
		print("Game is ending")
		return

	def onBeginGameTurn(self, argsList):
		'Called at the beginning of the end of each turn'
		iGameTurn = argsList[0]
		CvTopCivs.CvTopCivs().turnChecker(iGameTurn)

	def onEndGameTurn(self, argsList):
		'Called at the end of the end of each turn'
		iGameTurn = argsList[0]
		
	def onBeginPlayerTurn(self, argsList):
		'Called at the beginning of a players turn'
		iGameTurn, iPlayer = argsList	
		pPlayer = gc.getPlayer(iPlayer)
		
		pPlayer = gc.getPlayer(iPlayer)
# Rise of Mankind 2.7 start - world bank national wonder
		if gc.getTeam(pPlayer.getTeam()).isHasTech(gc.getInfoTypeForString('TECH_APPLIED_ECONOMICS')):
			b_WorldBank = gc.getInfoTypeForString("BUILDING_WORLD_BANK")
			obsoleteTech = gc.getBuildingInfo(b_WorldBank).getObsoleteTech()
			if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
				for iCity in range(pPlayer.getNumCities()):
					ppCity = pPlayer.getCity(iCity)
					if ppCity.getNumActiveBuilding(b_WorldBank) == true:
						#pPID = pPlayer.getID()
						iGold = pPlayer.getGold( )
						pPlayer.changeGold( iGold//50 )
# Rise of Mankind 2.7 end
		
		#Inquisition Mod
		
#		BugUtil.info("RoM - Begin Player turn, do religious victory checks")
		# Code to determine if all cities have 6 "Religious Unity" Prerequisites by Orion Veteran
		# Special thanks to STO for providing the code strategy - Great Job! 
		# 1. A player must have an official State Religion
		# 2. A player must have the Holy City for the official State Religion	
		# 3. A player must have built the Shrine for the official State Religion			
		# 4. 80% a player's cities must have the official state religion established.
		# 5. All of a player's cities must not have any non-state religions established.
		# 6. Religious influence must be at least 80%
		
		#Is the "Religious" victory game option set to True?
		if CyGame().isVictoryValid(gc.getInfoTypeForString("VICTORY_RELIGIOUS")) :
			if (pPlayer.getNumCities() > 0) and (CyGame().getNumCivCities() > 2 * CyGame().countCivPlayersAlive()): # be sure the game is advanced enough to test a religious victory
				bUnity = True
				iStateReligion = pPlayer.getStateReligion()
			
				if iStateReligion == -1 :
					# No State Religion was Found - Prerequisite 1
					#CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'No State Religion!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),0,0,False,False)
					bUnity = False
				
				if bUnity :				
					if not pPlayer.hasHolyCity(iStateReligion) :
						# No Holy City was Found for the official state religion - Prerequisite 2
						#CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'Holy City Not Found!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),0,0,False,False)
						bUnity = False
			
				if bUnity :
					bHasShrine = False
					MyShrine = CvUtil.CvGetStateReligionShrine(iPlayer)
					#CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'Test','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),0,0,False,False)

					for iCity in range(pPlayer.getNumCities()):
						CVpCity = pPlayer.getCity(iCity)
						if CVpCity.isHasBuilding(gc.getInfoTypeForString(MyShrine)):
							# No official State Religion Shrine was found for the official state religion - Prerequisite 3
							bHasShrine = True
							break
							
					if not bHasShrine :
						#CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'No State Shrine was Found!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),0,0,False,False)
						bUnity = False
						
				if bUnity :
					#RevolutionDCM - relaxation of prerequisite 4 because of the practical difficulty in spreading state religion to every satellite settlement
					maxNonStatePercentThreshold = 20
					nonStateReligionCount = 0	
					numCities = len(PyHelpers.PyPlayer(iPlayer).getCityList())

					for pyCity in PyHelpers.PyPlayer(iPlayer).getCityList() :
						if not bUnity :
							break
				
						for iReligionLoop in range(gc.getNumReligionInfos()):
							if iReligionLoop == iStateReligion :
								if not pyCity.hasReligion(iReligionLoop) :
									# City does not have the official state religion - Prerequisite 4
									#CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'No State Religion founded in City!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),0,0,False,False)
									nonStateReligionCount += 1
								nonStateReligionPercent = 100 * nonStateReligionCount / numCities
								if nonStateReligionPercent > maxNonStatePercentThreshold:
									bUnity = False
									break
							elif pyCity.hasReligion(iReligionLoop) :
								# City has a non-State religion - Prerequisite 5
								if not CvUtil.isOC_INQISITOR_CAN_REMOVE_HOLY_CITY():
									if not CvUtil.isHoly(pyCity):
										bUnity = False
										break
								else:	
									#CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'Found non-State Religion!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),0,0,False,False)
									bUnity = False
									break
								
				if bUnity :
					if CyGame().calculateReligionPercent(iStateReligion) < 75 :
						# Religeous influence is less than 75% - Prerequisite 6
						#CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'No Religion influence!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),0,0,False,False)
						bUnity = False							

				if bUnity :
					# The player has met all 5 prerequsites to achieve a valid religeous victory
					#CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'Victory!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),0,0,False,False)
					CyGame().setWinner(pPlayer.getTeam(), gc.getInfoTypeForString("VICTORY_RELIGIOUS"))
# End Inquisition Mod

# Rise of Mankind 2.6 - Crusade Start ##
#		test = ( iGameTurn % 14 )
#		BugUtil.info ( "crusade test: %d", test) 
		b_Crusade = self.iBUILDING_CRUSADE
		obsoleteTech = gc.getBuildingInfo(b_Crusade).getObsoleteTech()

		if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == false or obsoleteTech == -1 ):
#			BugUtil.info("RoM - crusader spawn, tech check passed")
			for iCity in range(pPlayer.getNumCities()):
				ppCity = pPlayer.getCity(iCity)
				if ppCity.getNumActiveBuilding(b_Crusade) == true:

					iX = ppCity.getX()
					iY = ppCity.getY()
					u_crusader = gc.getInfoTypeForString( 'UNIT_CRUSADER' )

					estiEnd = CyGame().getEstimateEndTurn()
					if ( estiEnd >= 1800 ):
						if ( iGameTurn % 16 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )
					elif ( estiEnd >= 1400 ):
						if ( iGameTurn % 14 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )
					elif ( estiEnd >= 1000 ):
						if ( iGameTurn % 12 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )
					elif ( estiEnd >= 700 ):
						if ( iGameTurn % 8 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )
					elif ( estiEnd >= 500 ):
						if ( iGameTurn % 6 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )
					elif ( estiEnd >= 300 ):
						if ( iGameTurn % 4 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )
					else:
						if ( iGameTurn % 4 ) == 0:
							for i in range(1):
								pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )

# Rise of Mankind 2.6 - Crusade End #

		
	def onEndPlayerTurn(self, argsList):
		'Called at the end of a players turn'
		iGameTurn, iPlayer = argsList
		
		if (gc.getGame().getElapsedGameTurns() == 1):
			if (gc.getPlayer(iPlayer).isHuman()):
				if (gc.getPlayer(iPlayer).canRevolution(0)):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_CHANGECIVIC)
					popupInfo.addPopup(iPlayer)
		
		CvAdvisorUtils.resetAdvisorNags()
		CvAdvisorUtils.endTurnFeats(iPlayer)

	def onEndTurnReady(self, argsList):
		iGameTurn = argsList[0]

	def onFirstContact(self, argsList):
		'Contact'
		iTeamX,iHasMetTeamY = argsList
		if (not self.__LOG_CONTACT):
			return
		CvUtil.pyPrint('Team %d has met Team %d' %(iTeamX, iHasMetTeamY))
	
	def onCombatResult(self, argsList):
		'Combat Result'
		pWinner,pLoser = argsList
		playerX = PyPlayer(pWinner.getOwner())
		unitX = PyInfo.UnitInfo(pWinner.getUnitType())
		playerY = PyPlayer(pLoser.getOwner())
		unitY = PyInfo.UnitInfo(pLoser.getUnitType())
		if (not self.__LOG_COMBAT):
			return
		if playerX and playerX and unitX and playerY:
			CvUtil.pyPrint('Player %d Civilization %s Unit %s has defeated Player %d Civilization %s Unit %s' 
				%(playerX.getID(), playerX.getCivilizationName(), unitX.getDescription(), 
				playerY.getID(), playerY.getCivilizationName(), unitY.getDescription()))

	def onCombatLogCalc(self, argsList):
		'Combat Result'	
		genericArgs = argsList[0][0]
		cdAttacker = genericArgs[0]
		cdDefender = genericArgs[1]
		iCombatOdds = genericArgs[2]
		CvUtil.combatMessageBuilder(cdAttacker, cdDefender, iCombatOdds)
		
	def onCombatLogHit(self, argsList):
		'Combat Message'
		global gCombatMessages, gCombatLog
		genericArgs = argsList[0][0]
		cdAttacker = genericArgs[0]
		cdDefender = genericArgs[1]
		iIsAttacker = genericArgs[2]
		iDamage = genericArgs[3]
		
		if cdDefender.eOwner == cdDefender.eVisualOwner:
			szDefenderName = gc.getPlayer(cdDefender.eOwner).getNameKey()
		else:
			szDefenderName = localText.getText("TXT_KEY_TRAIT_PLAYER_UNKNOWN", ())
		if cdAttacker.eOwner == cdAttacker.eVisualOwner:
			szAttackerName = gc.getPlayer(cdAttacker.eOwner).getNameKey()
		else:
			szAttackerName = localText.getText("TXT_KEY_TRAIT_PLAYER_UNKNOWN", ())

		if (iIsAttacker == 0):
			combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_HIT", (szDefenderName, cdDefender.sUnitName, iDamage, cdDefender.iCurrHitPoints, cdDefender.iMaxHitPoints))
			CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
			CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
			if (cdDefender.iCurrHitPoints <= 0):
				combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_DEFEATED", (szAttackerName, cdAttacker.sUnitName, szDefenderName, cdDefender.sUnitName))
				CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
				CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
		elif (iIsAttacker == 1):
			combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_HIT", (szAttackerName, cdAttacker.sUnitName, iDamage, cdAttacker.iCurrHitPoints, cdAttacker.iMaxHitPoints))
			CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
			CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
			if (cdAttacker.iCurrHitPoints <= 0):
				combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_DEFEATED", (szDefenderName, cdDefender.sUnitName, szAttackerName, cdAttacker.sUnitName))
				CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
				CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)

	def onImprovementBuilt(self, argsList):
		'Improvement Built'
		iImprovement, iX, iY = argsList
###AdvancedNukesbegin###
		pPlot = CyMap().plot(iX,iY)
		if(iImprovement==gc.getInfoTypeForString('IMPROVEMENT_SANITIZE_WATER')):
			pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)
			pPlot.setImprovementType(-1)
###AdvancedNukesend###
		if (not self.__LOG_IMPROVEMENT):
			return
		CvUtil.pyPrint('Improvement %s was built at %d, %d'
			%(PyInfo.ImprovementInfo(iImprovement).getDescription(), iX, iY))

	def onImprovementDestroyed(self, argsList):
		'Improvement Destroyed'
		iImprovement, iOwner, iX, iY = argsList
		if (not self.__LOG_IMPROVEMENT):
			return
		CvUtil.pyPrint('Improvement %s was Destroyed at %d, %d'
			%(PyInfo.ImprovementInfo(iImprovement).getDescription(), iX, iY))

	def onRouteBuilt(self, argsList):
		'Route Built'
		iRoute, iX, iY = argsList
		if (not self.__LOG_IMPROVEMENT):
			return
		CvUtil.pyPrint('Route %s was built at %d, %d'
			%(gc.getRouteInfo(iRoute).getDescription(), iX, iY))

	def onPlotRevealed(self, argsList):
		'Plot Revealed'
		pPlot = argsList[0]
		iTeam = argsList[1]

	def onPlotFeatureRemoved(self, argsList):
		'Plot Revealed'
		pPlot = argsList[0]
		iFeatureType = argsList[1]
		pCity = argsList[2] # This can be null

	def onPlotPicked(self, argsList):
		'Plot Picked'
		pPlot = argsList[0]
		CvUtil.pyPrint('Plot was picked at %d, %d'
			%(pPlot.getX(), pPlot.getY()))

	def onNukeExplosion(self, argsList):
		'Nuke Explosion'
		pPlot, pNukeUnit = argsList
		CvUtil.pyPrint('Nuke detonated at %d, %d'
			%(pPlot.getX(), pPlot.getY()))
###AdvancedNukesbegin###		
		if (pNukeUnit is not None and pNukeUnit.getUnitType() == gc.getInfoTypeForString('UNIT_FUSION_NUKE')):

                        
                        iX = pPlot.getX()
                        iY = pPlot.getY()

			for iXLoop in range(iX - 0, iX + 1, 1):
				for iYLoop in range(iY - 0, iY + 1, 1):
						pPlot = CyMap().plot(iXLoop, iYLoop)
						if (( pPlot.isPeak()==true  ) or (pPlot.isHills()==true)):
							pPlot.setPlotType(PlotTypes.PLOT_LAND, True, True)
 						pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)

		if (pNukeUnit is not None and pNukeUnit.getUnitType() == gc.getInfoTypeForString('UNIT_FUSION_NOVA')):

                        
                        iX = pPlot.getX()
                        iY = pPlot.getY()

			for iXLoop in range(iX - 1, iX + 2, 1):
				for iYLoop in range(iY - 1, iY + 2, 1):
						pPlot = CyMap().plot(iXLoop, iYLoop)
						if (( pPlot.isPeak()==true  ) or (pPlot.isHills()==true)):
							pPlot.setPlotType(PlotTypes.PLOT_LAND, True, True)
 						pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_COAST" ), 1, 1)

		if (pNukeUnit is not None and pNukeUnit.getUnitType() == gc.getInfoTypeForString('UNIT_POISON_NUKE')):

                        
                        iX = pPlot.getX()
                        iY = pPlot.getY()

			for iXLoop in range(iX - 1, iX + 2, 1):
				for iYLoop in range(iY - 1, iY + 2, 1):
						pPlot = CyMap().plot(iXLoop, iYLoop)
						pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_BIOGAS" ), 1)
						if ( pPlot.isWater()==true  ):
 							pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_SLIMY_COAST" ), 1, 1)

		if (pNukeUnit is not None and pNukeUnit.getUnitType() == gc.getInfoTypeForString('UNIT_POISON_NOVA')):

                        
                        iX = pPlot.getX()
                        iY = pPlot.getY()

			for iXLoop in range(iX - 5, iX + 6, 1):
				for iYLoop in range(iY - 5, iY + 6, 1):
						pPlot = CyMap().plot(iXLoop, iYLoop)
						pPlot.setFeatureType(gc.getInfoTypeForString( "FEATURE_PLAGUEGAS" ), 1)
						if ( pPlot.isWater()==true  ):
							pPlot.setTerrainType(gc.getInfoTypeForString( "TERRAIN_SLIMY_OCEAN" ), 1, 1)
###AdvancedNukesend###

	def onGotoPlotSet(self, argsList):
		'Nuke Explosion'
		pPlot, iPlayer = argsList

	def onBuildingBuilt(self, argsList):
		'Building Completed'
		pCity, iBuildingType = argsList
		game = gc.getGame()
# Rise of Mankind 2.7 - Djenne Start

		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_DJENNE' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			pTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			tt_desert = gc.getInfoTypeForString( 'TERRAIN_DESERT' )

			for iXLoop in range(iX - 2, iX + 3, 1):
				for iYLoop in range(iY - 2, iY + 3, 1):
					pPlot = CyMap().plot(iXLoop, iYLoop)
					if ( pPlot.isPlayerCityRadius(pPID)==true ):
						if ( pPlot.getTeam()==pTID ):
							if ( pPlot.getTerrainType()==tt_desert ):
								CyGame().setPlotExtraYield(iXLoop, iYLoop, YieldTypes.YIELD_COMMERCE, 2)
			
			
			CyInterface().addMessage(pPID,false,15,CyTranslator().getText("TXT_KEY_DJENNE_PYTHON",()),'',0,'Art/Interface/Buttons/Buildings/Djenne.dds',ColorTypes(44), iX, iY, True,True)
# Rise of Mankind 2.7 - Djenne end

		
# Rise of Mankind 2.7 start - world bank national wonder
		if iBuildingType == gc.getInfoTypeForString( 'BUILDING_WORLD_BANK' ):

			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			pPID = pPlayer.getID()
			iGold = pPlayer.getGold( )
			pPlayer.changeGold( iGold//2 )
# Rise of Mankind 2.7 end
		
		if ((not gc.getGame().isNetworkMultiPlayer()) and (pCity.getOwner() == gc.getGame().getActivePlayer()) and isWorldWonderClass(gc.getBuildingInfo(iBuildingType).getBuildingClassType())):
			# If this is a wonder...
			popupInfo = CyPopupInfo()
			popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
			popupInfo.setData1(iBuildingType)
			popupInfo.setData2(pCity.getID())
			popupInfo.setData3(0)
			popupInfo.setText(u"showWonderMovie")
			popupInfo.addPopup(pCity.getOwner())

		CvAdvisorUtils.buildingBuiltFeats(pCity, iBuildingType)

# Rise of Mankind 2.63 start - building upgrade optimization
		# Building upgrade line check
		if self.aiBuildingUpgradeLine_Upgrade_InfoTypeNum.count(iBuildingType) > 0:	
			aiBuildingUpgradeLine_Upgrade_InfoTypeNum_Temp = self.aiBuildingUpgradeLine_Upgrade_InfoTypeNum[:]
			aiBuildingUpgradeLine_Obsolete_InfoTypeNum_Temp = self.aiBuildingUpgradeLine_Obsolete_InfoTypeNum[:]
			while aiBuildingUpgradeLine_Upgrade_InfoTypeNum_Temp.count(iBuildingType) > 0:
				iBuildingUpgradeLine_Item_Index = aiBuildingUpgradeLine_Upgrade_InfoTypeNum_Temp.index(iBuildingType)
				aiBuildingUpgradeLine_Upgrade_InfoTypeNum_Temp.remove(iBuildingType)
				pCity.setNumRealBuilding(aiBuildingUpgradeLine_Obsolete_InfoTypeNum_Temp.pop(iBuildingUpgradeLine_Item_Index), False)

# Rise of Mankind 2.63 end
		
# Rise of Mankind 2.6 - Crusade Start #

		# Crusade check
		if iBuildingType == self.iBUILDING_CRUSADE:
			pPlayer = gc.getPlayer(pCity.plot().getOwner())
			iPID = pPlayer.getID()
			iTID = pPlayer.getTeam()
			iX = pCity.getX()
			iY = pCity.getY()
			u_crusader = gc.getInfoTypeForString( 'UNIT_CRUSADER' )

			for i in range(1):
				pNewUnit = pPlayer.initUnit( u_crusader, iX, iY, UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.NO_DIRECTION )

# Rise of Mankind 2.6 - Crusade End ###

		if (not self.__LOG_BUILDING):
			return
		CvUtil.pyPrint('%s was finished by Player %d Civilization %s' 
			%(PyInfo.BuildingInfo(iBuildingType).getDescription(), pCity.getOwner(), gc.getPlayer(pCity.getOwner()).getCivilizationDescription(0)))
	
	def onProjectBuilt(self, argsList):
		'Project Completed'
		pCity, iProjectType = argsList
		game = gc.getGame()
		if ((not gc.getGame().isNetworkMultiPlayer()) and (pCity.getOwner() == gc.getGame().getActivePlayer())):
			popupInfo = CyPopupInfo()
			popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
			popupInfo.setData1(iProjectType)
			popupInfo.setData2(pCity.getID())
			popupInfo.setData3(2)
			popupInfo.setText(u"showWonderMovie")
			popupInfo.addPopup(pCity.getOwner())
				
	def onSelectionGroupPushMission(self, argsList):
		'selection group mission'
		eOwner = argsList[0]
		eMission = argsList[1]
		iNumUnits = argsList[2]
		listUnitIds = argsList[3]
		
		if (not self.__LOG_PUSH_MISSION):
			return
		if pHeadUnit:
			CvUtil.pyPrint("Selection Group pushed mission %d" %(eMission))
	
	def onUnitMove(self, argsList):
		'unit move'
		pPlot,pUnit,pOldPlot = argsList
		player = PyPlayer(pUnit.getOwner())
		unitInfo = PyInfo.UnitInfo(pUnit.getUnitType())
		if (not self.__LOG_MOVEMENT):
			return
		if player and unitInfo:
			CvUtil.pyPrint('Player %d Civilization %s unit %s is moving to %d, %d' 
				%(player.getID(), player.getCivilizationName(), unitInfo.getDescription(), 
				pUnit.getX(), pUnit.getY()))

	def onUnitSetXY(self, argsList):
		'units xy coords set manually'
		pPlot,pUnit = argsList
		player = PyPlayer(pUnit.getOwner())
		unitInfo = PyInfo.UnitInfo(pUnit.getUnitType())
		if (not self.__LOG_MOVEMENT):
			return
		
	def onUnitCreated(self, argsList):
		'Unit Completed'
		unit = argsList[0]
		player = PyPlayer(unit.getOwner())
		if (not self.__LOG_UNITBUILD):
			return

	def onUnitBuilt(self, argsList):
		'Unit Completed'
		city = argsList[0]
		unit = argsList[1]
		player = PyPlayer(city.getOwner())

		CvAdvisorUtils.unitBuiltFeats(city, unit)
# Rise of Mankind 2.5
		# if player has Technocracy civic active, give free promotion to unit
		iPlayer = gc.getPlayer(city.getOwner())
		if gc.getTeam(iPlayer.getTeam()).isHasTech(gc.getInfoTypeForString('TECH_KNOWLEDGE_MANAGEMENT')):
			iFutureCivicOption = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo,gc.getNumCivicOptionInfos(),'CIVICOPTION_FUTURE')
			iTechnocracy = CvUtil.findInfoTypeNum(gc.getCivicInfo,gc.getNumCivicInfos(),'CIVIC_TECHNOCRACY')

			iFutureCivic = iPlayer.getCivics(iFutureCivicOption)

			if (iFutureCivic == iTechnocracy):
				iSensors =  CvUtil.findInfoTypeNum(gc.getPromotionInfo, gc.getNumPromotionInfos(),'PROMOTION_SENSORS')
				if ( gc.getTeam(iPlayer.getTeam()).isHasTech(gc.getInfoTypeForString("TECH_SMART_DUST")) == true ):
					if (unit.isPromotionValid(iSensors)):
						unit.setHasPromotion(iSensors, True)
# Rise of Mankind 2.5

		
		if (not self.__LOG_UNITBUILD):
			return
		CvUtil.pyPrint('%s was finished by Player %d Civilization %s' 
			%(PyInfo.UnitInfo(unit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))
	
	def onUnitKilled(self, argsList):
		'Unit Killed'
		unit, iAttacker = argsList
		player = PyPlayer(unit.getOwner())
		attacker = PyPlayer(iAttacker)
		if (not self.__LOG_UNITKILLED):
			return
		CvUtil.pyPrint('Player %d Civilization %s Unit %s was killed by Player %d' 
			%(player.getID(), player.getCivilizationName(), PyInfo.UnitInfo(unit.getUnitType()).getDescription(), attacker.getID()))

	def onUnitLost(self, argsList):
		'Unit Lost'
		unit = argsList[0]
		player = PyPlayer(unit.getOwner())
		if (not self.__LOG_UNITLOST):
			return
		CvUtil.pyPrint('%s was lost by Player %d Civilization %s' 
			%(PyInfo.UnitInfo(unit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))
	
	def onUnitPromoted(self, argsList):
		'Unit Promoted'
		pUnit, iPromotion = argsList
		player = PyPlayer(pUnit.getOwner())
		if (not self.__LOG_UNITPROMOTED):
			return
		CvUtil.pyPrint('Unit Promotion Event: %s - %s' %(player.getCivilizationName(), pUnit.getName(),))
	
	def onUnitSelected(self, argsList):
		'Unit Selected'
		unit = argsList[0]
		player = PyPlayer(unit.getOwner())
		if (not self.__LOG_UNITSELECTED):
			return
		CvUtil.pyPrint('%s was selected by Player %d Civilization %s' 
			%(PyInfo.UnitInfo(unit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))
	
	def onUnitRename(self, argsList):
		'Unit is renamed'
		pUnit = argsList[0]
		if (pUnit.getOwner() == gc.getGame().getActivePlayer()):
			self.__eventEditUnitNameBegin(pUnit)
	
	def onUnitPillage(self, argsList):
		'Unit pillages a plot'
		pUnit, iImprovement, iRoute, iOwner = argsList
		iPlotX = pUnit.getX()
		iPlotY = pUnit.getY()
		pPlot = CyMap().plot(iPlotX, iPlotY)
		
		if (not self.__LOG_UNITPILLAGE):
			return
		CvUtil.pyPrint("Player %d's %s pillaged improvement %d and route %d at plot at (%d, %d)" 
			%(iOwner, PyInfo.UnitInfo(pUnit.getUnitType()).getDescription(), iImprovement, iRoute, iPlotX, iPlotY))
	
	def onUnitSpreadReligionAttempt(self, argsList):
		'Unit tries to spread religion to a city'
		pUnit, iReligion, bSuccess = argsList
		
		iX = pUnit.getX()
		iY = pUnit.getY()
		pPlot = CyMap().plot(iX, iY)
		pCity = pPlot.getPlotCity()
	
	def onUnitGifted(self, argsList):
		'Unit is gifted from one player to another'
		pUnit, iGiftingPlayer, pPlotLocation = argsList
	
	def onUnitBuildImprovement(self, argsList):
		'Unit begins enacting a Build (building an Improvement or Route)'
		pUnit, iBuild, bFinished = argsList

	def onGoodyReceived(self, argsList):
		'Goody received'
		iPlayer, pPlot, pUnit, iGoodyType = argsList
		if (not self.__LOG_GOODYRECEIVED):
			return
		CvUtil.pyPrint('%s received a goody' %(gc.getPlayer(iPlayer).getCivilizationDescription(0)),)
	
	def onGreatPersonBorn(self, argsList):
		'Unit Promoted'
		pUnit, iPlayer, pCity = argsList
		player = PyPlayer(iPlayer)
		if pUnit.isNone() or pCity.isNone():
			return
		if (not self.__LOG_GREATPERSON):
			return
		CvUtil.pyPrint('A %s was born for %s in %s' %(pUnit.getName(), player.getCivilizationName(), pCity.getName()))
	
	def onTechAcquired(self, argsList):
		'Tech Acquired'
		iTechType, iTeam, iPlayer, bAnnounce = argsList
		# Note that iPlayer may be NULL (-1) and not a refer to a player object
		# Inquisition Mod
		rSlotAvailable = False
		xChooseReligion = False
		iRCount = 0
		
	
		#pHolyCity = False
		# End Inquisition Mod
		# Show tech splash when applicable
		if (iPlayer > -1 and bAnnounce and not CyInterface().noTechSplash()):
			if (gc.getGame().isFinalInitialized() and not gc.getGame().GetWorldBuilderMode()):
				if ((not gc.getGame().isNetworkMultiPlayer()) and (iPlayer == gc.getGame().getActivePlayer())):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
					popupInfo.setData1(iTechType)
					popupInfo.setText(u"showTechSplash")
					popupInfo.addPopup(iPlayer)
					
		# Inquisition Mod
		CvPlayer = gc.getPlayer(iPlayer)
		
		# Is Tech a Religious Tech?
		if CvUtil.CvIsReligiousTech(iTechType):
#			BugUtil.info("Inquisition, religious tech acquired")
			# If game option is Limited Religions
			if RevDCMOpt.isOC_LIMITED_RELIGIONS():
				if not CvPlayer.isBarbarian() and CvPlayer.isAlive():
					#if iPlayer > -1 and not CvPlayer.isNone() and not CvPlayer.isBarbarian() and CvPlayer.isAlive():
					#CvGameUtils.LR_iPlayer = iPlayer
					# Does the player have a Holy City?
					if not CvUtil.OwnsHolyCity(iPlayer):
						# Loop through Religions
						for iReligion in range(gc.getNumReligionInfos()):
							# If the religion has not been founded
							if not gc.getGame().isReligionFounded(iReligion):
								# If game option is not set to choose religions
								if not CyGame().isOption(GameOptionTypes.GAMEOPTION_PICK_RELIGION):
									if gc.getReligionInfo(iReligion).getTechPrereq() == iTechType:
										gc.getPlayer(iPlayer).foundReligion(iReligion,iReligion,True)
										#CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'Message 1','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),0,0,False,False)
										break
								# If game option is set to choose religions
								elif CyGame().isOption(GameOptionTypes.GAMEOPTION_PICK_RELIGION):
									rSlotAvailable = True
									break
						
						#  There is one or more unfounded Religions
						if rSlotAvailable:
							if CvUtil.CvGetTotalReligionsFounded() < 3:
								if iTechType == gc.getInfoTypeForString("TECH_CEREMONIAL_BURIAL") or iTechType == gc.getInfoTypeForString("TECH_POLYTHEISM") or iTechType == gc.getInfoTypeForString("TECH_DUALISM"):	
									xChooseReligion = True
							elif CvUtil.CvGetTotalReligionsFounded() < 5:
								if iTechType == gc.getInfoTypeForString("TECH_AESTHETICS") or iTechType == gc.getInfoTypeForString("TECH_MONOTHEISM"):
									xChooseReligion = True
							elif CvUtil.CvGetTotalReligionsFounded() < 7:
								if iTechType == gc.getInfoTypeForString("TECH_CODE_OF_LAWS") or iTechType == gc.getInfoTypeForString("TECH_MEDITATION"):
									xChooseReligion = True
							elif CvUtil.CvGetTotalReligionsFounded() < 9:
								if iTechType == gc.getInfoTypeForString("TECH_THEOLOGY") or iTechType == gc.getInfoTypeForString("TECH_CALENDAR"):	
									xChooseReligion = True
							elif CvUtil.CvGetTotalReligionsFounded() < 11:
								if iTechType == gc.getInfoTypeForString("TECH_EDUCATION") or iTechType == gc.getInfoTypeForString("TECH_PHILOSOPHY"):	
									xChooseReligion = True
						
						
						if xChooseReligion:
							if not CvPlayer.isHuman():
								if not CvUtil.OwnsHolyCity(iPlayer):
# rise of mankind 2.6 start - AI player chooses favorite religion rather than random one
									eFavorite = gc.getLeaderHeadInfo(CvPlayer.getLeaderType()).getFavoriteReligion()
									if eFavorite > -1 and not CyGame().isReligionFounded(eFavorite):
										gc.getPlayer(iPlayer).foundReligion(eFavorite,eFavorite,True)
									else:	
										gc.getPlayer(iPlayer).foundReligion(iReligion,iReligion,True)
# rise of mankind 2.6 end
							else:
								# Create Religion Selection Popup
								popupInfo = CyPopupInfo()
								popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
								popupInfo.setData1(iPlayer)
								popupInfo.setText(CyTranslator().getText(("TXT_KEY_FOUNDED_RELIGION"),()))
								
								# Find all unfounded religions
								for iNewReligion in range(gc.getNumReligionInfos()):
									if not gc.getGame().isReligionFounded(iNewReligion):
										popupInfo.addPythonButton(gc.getReligionInfo(iNewReligion).getDescription(), gc.getReligionInfo(iNewReligion).getButton())
										iRCount = iRCount + 1
										if iRCount == gc.getNumReligionInfos() - CvUtil.CvGetTotalReligionsFounded():
											# Display popup for human player to selected Religion
											popupInfo.addPopup(1)
											#CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'Message C','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),0,0,False,False)
											break
											# End Inquisition Mod
		if (not self.__LOG_TECH):
			return
		CvUtil.pyPrint('%s was finished by Team %d'
			%(PyInfo.TechnologyInfo(iTechType).getDescription(), iTeam))
	
	def onTechSelected(self, argsList):
		'Tech Selected'
		iTechType, iPlayer = argsList
		if (not self.__LOG_TECH):
			return
		CvUtil.pyPrint('%s was selected by Player %d' %(PyInfo.TechnologyInfo(iTechType).getDescription(), iPlayer))
	
	def onReligionFounded(self, argsList):
		'Religion Founded'
		iReligion, iFounder = argsList
		player = PyPlayer(iFounder)
		
		iCityId = gc.getGame().getHolyCity(iReligion).getID()
		if (gc.getGame().isFinalInitialized() and not gc.getGame().GetWorldBuilderMode()):
			if ((not gc.getGame().isNetworkMultiPlayer()) and (iFounder == gc.getGame().getActivePlayer())):
				popupInfo = CyPopupInfo()
				popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
				popupInfo.setData1(iReligion)
				popupInfo.setData2(iCityId)
				popupInfo.setData3(1)
				popupInfo.setText(u"showWonderMovie")
				popupInfo.addPopup(iFounder)
		
		if (not self.__LOG_RELIGION):
			return
		CvUtil.pyPrint('Player %d Civilization %s has founded %s'
			%(iFounder, player.getCivilizationName(), gc.getReligionInfo(iReligion).getDescription()))

	def onReligionSpread(self, argsList):
		'Religion Has Spread to a City'
		iReligion, iOwner, pSpreadCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has spread to Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pSpreadCity.getName()))

	def onReligionRemove(self, argsList):
		'Religion Has been removed from a City'
		iReligion, iOwner, pRemoveCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has been removed from Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pRemoveCity.getName()))
				
	def onCorporationFounded(self, argsList):
		'Corporation Founded'
		iCorporation, iFounder = argsList
		player = PyPlayer(iFounder)
		
		if (not self.__LOG_RELIGION):
			return
		CvUtil.pyPrint('Player %d Civilization %s has founded %s'
			%(iFounder, player.getCivilizationName(), gc.getCorporationInfo(iCorporation).getDescription()))

	def onCorporationSpread(self, argsList):
		'Corporation Has Spread to a City'
		iCorporation, iOwner, pSpreadCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has spread to Player %d Civilization %s city of %s'
			%(gc.getCorporationInfo(iCorporation).getDescription(), iOwner, player.getCivilizationName(), pSpreadCity.getName()))

	def onCorporationRemove(self, argsList):
		'Corporation Has been removed from a City'
		iCorporation, iOwner, pRemoveCity = argsList
		player = PyPlayer(iOwner)
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has been removed from Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pRemoveCity.getName()))
				
	def onGoldenAge(self, argsList):
		'Golden Age'
		iPlayer = argsList[0]
		player = PyPlayer(iPlayer)
		if (not self.__LOG_GOLDENAGE):
			return
		CvUtil.pyPrint('Player %d Civilization %s has begun a golden age'
			%(iPlayer, player.getCivilizationName()))

	def onEndGoldenAge(self, argsList):
		'End Golden Age'
		iPlayer = argsList[0]
		player = PyPlayer(iPlayer)
		if (not self.__LOG_ENDGOLDENAGE):
			return
		CvUtil.pyPrint('Player %d Civilization %s golden age has ended'
			%(iPlayer, player.getCivilizationName()))

	def onChangeWar(self, argsList):
		'War Status Changes'
		bIsWar = argsList[0]
		iTeam = argsList[1]
		iRivalTeam = argsList[2]
		if (not self.__LOG_WARPEACE):
			return
		if (bIsWar):
			strStatus = "declared war"
		else:
			strStatus = "declared peace"
		CvUtil.pyPrint('Team %d has %s on Team %d'
			%(iTeam, strStatus, iRivalTeam))
	
	def onChat(self, argsList):
		'Chat Message Event'
		chatMessage = "%s" %(argsList[0],)
		
	def onSetPlayerAlive(self, argsList):
		'Set Player Alive Event'
		iPlayerID = argsList[0]
		bNewValue = argsList[1]
		CvUtil.pyPrint("Player %d's alive status set to: %d" %(iPlayerID, int(bNewValue)))
		
	def onPlayerChangeStateReligion(self, argsList):
		'Player changes his state religion'
		iPlayer, iNewReligion, iOldReligion = argsList
		
	def onPlayerGoldTrade(self, argsList):
		'Player Trades gold to another player'
		iFromPlayer, iToPlayer, iGoldAmount = argsList
		
	def onCityBuilt(self, argsList):
		'City Built'
		city = argsList[0]
		if (city.getOwner() == gc.getGame().getActivePlayer()):
			self.__eventEditCityNameBegin(city, False)	
		CvUtil.pyPrint('City Built Event: %s' %(city.getName()))
		
	def onCityRazed(self, argsList):
		'City Razed'
		city, iPlayer = argsList
		iOwner = city.findHighestCulture()
		iGoldTotal = 0
		Respawn = False
		pPlayer = gc.getPlayer(iPlayer)
		iStateReligion = pPlayer.getStateReligion()
		
# Rise of Mankind 2.6 Start 
		self.iArcologyCityID = -1
		
		if city.getNumRealBuilding(gc.getInfoTypeForString("BUILDING_ARCOLOGY")) or city.getNumRealBuilding(gc.getInfoTypeForString("BUILDING_ARCOLOGY_SHIELDING")) or city.getNumRealBuilding(gc.getInfoTypeForString("BUILDING_ADVANCED_SHIELDING")):
			self.iArcologyCityID = city.getID()
		
		# Removes all non-state Religious Buildings from the City
		for iBuildingLoop in range(gc.getNumBuildingInfos( )):
			if city.isHasBuilding( iBuildingLoop ):
				pBuilding = gc.getBuildingInfo( iBuildingLoop )
				iRequiredReligion = pBuilding.getReligionType( )
				for iReligionLoop in range(gc.getNumReligionInfos()):
					if iReligionLoop != pPlayer.getStateReligion( ):
						if iRequiredReligion == iReligionLoop:
							if CvUtil.OCHasNonStateShrine(pPlayer, iBuildingLoop):
								# Greatly increases Anger for all AIs who share the religion of the destroyed Shrine
								CvUtil.AIAttitudeAdjustment(iPlayer, iRequiredReligion, -3 )
								# Add gold to treasury for pillaging the non-state Shrine
# Rise of Mankind 2.7 - adjustments to how much gold you get from purging non-state religions, 20% from Orion's Inquisition defaults
								iGoldTotal = iGoldTotal + 100 # Orion's Inquisition default 500
								CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'Destruction of a non-state Holy Shrine has angered AI Civs!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(7),0,0,False,False)
							else:
								# Increases Anger for all AIs who share the religion of a destroyed Temple, Monastery or Cathedral
								CvUtil.AIAttitudeAdjustment(iPlayer, iRequiredReligion, -1 )
								# Add gold to treasury for pillaging the non-state Religious Building									
								iGoldTotal = iGoldTotal + 50 # Orion's Inquisition default 250
							
		# Check default -- Does game option allow removal of a Holy City?
		if RevDCMOpt.isOC_RESPAWN_HOLY_CITIES():
			for iReligionLoop in range(gc.getNumReligionInfos()):
				# Make sure only non-state religions are selected
				if iReligionLoop != iStateReligion:
					# Is this a non-state Holy City?
					if city.isHolyCityByType( iReligionLoop ):
						# Increases Anger for all AIs who share the purged Holy City
						CvUtil.AIAttitudeAdjustment(iPlayer, iReligionLoop, -3 )
						# Remove non-state Holy City identifier
						gc.getGame( ).clearHolyCity( iReligionLoop )
						# Add gold to treasury for pillaging the non-state Holy City							
						iGoldTotal = iGoldTotal + 100 # Orion's Inquisition default 500
						# Increases Anger for all AIs who share the purged religion						
						CvUtil.AIAttitudeAdjustment(iPlayer, iReligionLoop, -1 )
						# Remove non-state religion
						city.setHasReligion(iReligionLoop, 0, 0, 0)						
						# Add gold to treasury to compensate for inquisition costs
						iGoldTotal = iGoldTotal + 20 # Orion's Inquisition default 100
						CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'Destruction of a non-state Holy City has angered AI Civs!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(7),0,0,False,False)
						# Is there at least 1 non Holy City remaining with HC Religion?  
						if CvUtil.findCityWithXReligion(iReligionLoop):
							# Set Minimum of Two Cities required for Respawning a Holy City
							minCitiesForInfluence = 2
							# Find player with the greatest Religious Influence for Religion 
							eBestPlayer = CvUtil.getPlayerWithMaxInfluenceForReligion(iReligionLoop, minCitiesForInfluence)	
							# If there is a player with enough Religious Influence for Religion
							if eBestPlayer >= 0:
								XpPlayer = gc.getPlayer(eBestPlayer)
								# Respawn the Holy City 
								CvUtil.ReSpawnHolyCity(XpPlayer, iReligionLoop)
					
					elif city.isHasReligion( iReligionLoop ):
						# Increases Anger for all AIs who share the purged religion						
						CvUtil.AIAttitudeAdjustment(iPlayer, iReligionLoop, -1 )
						# Add gold to treasury for pillaging 						
						iGoldTotal = iGoldTotal + 20 # Orion's Inquisition default 100
# Rise of Mankind 2.7 end
			# Total Pillage Reward
			pPlayer.changeGold(iGoldTotal)
# Rise of Mankind 2.6 end
			
		# Partisans!
		if city.getPopulation > 1 and iOwner != -1 and iPlayer != -1:
			owner = gc.getPlayer(iOwner)
			if not owner.isBarbarian() and owner.getNumCities() > 0:
				if gc.getTeam(owner.getTeam()).isAtWar(gc.getPlayer(iPlayer).getTeam()):
					if gc.getNumEventTriggerInfos() > 0: # prevents mods that don't have events from getting an error
						iEvent = CvUtil.findInfoTypeNum(gc.getEventTriggerInfo, gc.getNumEventTriggerInfos(),'EVENTTRIGGER_PARTISANS')
						if iEvent != -1 and gc.getGame().isEventActive(iEvent) and owner.getEventTriggerWeight(iEvent) < 0:
							triggerData = owner.initTriggeredData(iEvent, true, -1, city.getX(), city.getY(), iPlayer, city.getID(), -1, -1, -1, -1)
			
		CvUtil.pyPrint("City Razed Event: %s" %(city.getName(),))
			
	def onCityAcquired(self, argsList):
		'City Acquired'
		iPreviousOwner,iNewOwner,pCity,bConquest,bTrade = argsList
		CvUtil.pyPrint('City Acquired Event: %s' %(pCity.getName()))
	
	def onCityAcquiredAndKept(self, argsList):
		'City Acquired and Kept'
		iOwner,pCity = argsList
		CvUtil.pyPrint('City Acquired and Kept Event: %s' %(pCity.getName()))
	
	def onCityLost(self, argsList):
		'City Lost'
		city = argsList[0]
		player = PyPlayer(city.getOwner())
		
		# RoM Start
		if city.getID() == self.iArcologyCityID:
			city.plot().setImprovementType(gc.getInfoTypeForString("IMPROVEMENT_CITY_RUINS_ARCOLOGY"))
		# RoM end
		
		if (not self.__LOG_CITYLOST):
			return
		CvUtil.pyPrint('City %s was lost by Player %d Civilization %s' 
			%(city.getName(), player.getID(), player.getCivilizationName()))
	
	def onCultureExpansion(self, argsList):
		'City Culture Expansion'
		pCity = argsList[0]
		iPlayer = argsList[1]
		CvUtil.pyPrint("City %s's culture has expanded" %(pCity.getName(),))
	
	def onCityGrowth(self, argsList):
		'City Population Growth'
		pCity = argsList[0]
		iPlayer = argsList[1]
		CvUtil.pyPrint("%s has grown" %(pCity.getName(),))
	
	def onCityDoTurn(self, argsList):
		'City Production'
		pCity = argsList[0]
		iPlayer = argsList[1]

		CvAdvisorUtils.cityAdvise(pCity, iPlayer)
	
	def onCityBuildingUnit(self, argsList):
		'City begins building a unit'
		pCity = argsList[0]
		iUnitType = argsList[1]
		if (not self.__LOG_CITYBUILDING):
			return
		CvUtil.pyPrint("%s has begun building a %s" %(pCity.getName(),gc.getUnitInfo(iUnitType).getDescription()))
	
	def onCityBuildingBuilding(self, argsList):
		'City begins building a Building'
		pCity = argsList[0]
		iBuildingType = argsList[1]
		if (not self.__LOG_CITYBUILDING):
			return
		CvUtil.pyPrint("%s has begun building a %s" %(pCity.getName(),gc.getBuildingInfo(iBuildingType).getDescription()))
	
	def onCityRename(self, argsList):
		'City is renamed'
		pCity = argsList[0]
		if (pCity.getOwner() == gc.getGame().getActivePlayer()):
			self.__eventEditCityNameBegin(pCity, True)	
	
	def onCityHurry(self, argsList):
		'City is renamed'
		pCity = argsList[0]
		iHurryType = argsList[1]

	def onVictory(self, argsList):
		'Victory'
		iTeam, iVictory = argsList
		if (iVictory >= 0 and iVictory < gc.getNumVictoryInfos()):
			victoryInfo = gc.getVictoryInfo(int(iVictory))
			CvUtil.pyPrint("Victory!  Team %d achieves a %s victory"
				%(iTeam, victoryInfo.getDescription()))
	
	def onVassalState(self, argsList):
		'Vassal State'
		iMaster, iVassal, bVassal = argsList
		
		if (bVassal):
			CvUtil.pyPrint("Team %d becomes a Vassal State of Team %d"
				%(iVassal, iMaster))
		else:
			CvUtil.pyPrint("Team %d revolts and is no longer a Vassal State of Team %d"
				%(iVassal, iMaster))
	
	def onGameUpdate(self, argsList):
		'sample generic event, called on each game turn slice'
		genericArgs = argsList[0][0]	# tuple of tuple of my args
		turnSlice = genericArgs[0]
# Rise of Mankind 2.7
		OOSLogger.doGameUpdate()
# Rise of Mankind 2.7
		
	def onMouseEvent(self, argsList):
		'mouse handler - returns 1 if the event was consumed'
		eventType,mx,my,px,py,interfaceConsumed,screens = argsList
		if ( px!=-1 and py!=-1 ):
			if ( eventType == self.EventLButtonDown ):
				if (self.bAllowCheats and self.bCtrl and self.bAlt and CyMap().plot(px,py).isCity() and not interfaceConsumed):
					# Launch Edit City Event
					self.beginEvent( CvUtil.EventEditCity, (px,py) )
					return 1
				
				elif (self.bAllowCheats and self.bCtrl and self.bShift and not interfaceConsumed):
					# Launch Place Object Event
					self.beginEvent( CvUtil.EventPlaceObject, (px, py) )
					return 1
			
		if ( eventType == self.EventBack ):
			return CvScreensInterface.handleBack(screens)
		elif ( eventType == self.EventForward ):
			return CvScreensInterface.handleForward(screens)
		
		return 0

#################### TRIGGERED EVENTS ##################	
				
	def __eventEditCityNameBegin(self, city, bRename):
		popup = PyPopup.PyPopup(CvUtil.EventEditCityName, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setUserData((city.getID(), bRename))
		popup.setHeaderString(localText.getText("TXT_KEY_NAME_CITY", ()))
		popup.setBodyString(localText.getText("TXT_KEY_SETTLE_NEW_CITY_NAME", ()))
		popup.createEditBox(city.getName())
		popup.setEditBoxMaxCharCount( 15 )
		popup.launch()
	
	def __eventEditCityNameApply(self, playerID, userData, popupReturn):	
		'Edit City Name Event'
		iCityID = userData[0]
		bRename = userData[1]
		player = gc.getPlayer(playerID)
		city = player.getCity(iCityID)
		cityName = popupReturn.getEditBoxString(0)
		if (len(cityName) > 30):
			cityName = cityName[:30]
		city.setName(cityName, not bRename)

	def __eventEditCityBegin(self, argsList):
		'Edit City Event'
		px,py = argsList
		CvWBPopups.CvWBPopups().initEditCity(argsList)
	
	def __eventEditCityApply(self, playerID, userData, popupReturn):
		'Edit City Event Apply'
		if (getChtLvl() > 0):
			CvWBPopups.CvWBPopups().applyEditCity( (popupReturn, userData) )

	def __eventPlaceObjectBegin(self, argsList):
		'Place Object Event'
		CvDebugTools.CvDebugTools().initUnitPicker(argsList)
	
	def __eventPlaceObjectApply(self, playerID, userData, popupReturn):
		'Place Object Event Apply'
		if (getChtLvl() > 0):
			CvDebugTools.CvDebugTools().applyUnitPicker( (popupReturn, userData) )

	def __eventAwardTechsAndGoldBegin(self, argsList):
		'Award Techs & Gold Event'
		CvDebugTools.CvDebugTools().cheatTechs()
	
	def __eventAwardTechsAndGoldApply(self, playerID, netUserData, popupReturn):
		'Award Techs & Gold Event Apply'
		if (getChtLvl() > 0):
			CvDebugTools.CvDebugTools().applyTechCheat( (popupReturn) )
	
	def __eventShowWonderBegin(self, argsList):
		'Show Wonder Event'
		CvDebugTools.CvDebugTools().wonderMovie()
	
	def __eventShowWonderApply(self, playerID, netUserData, popupReturn):
		'Wonder Movie Apply'
		if (getChtLvl() > 0):
			CvDebugTools.CvDebugTools().applyWonderMovie( (popupReturn) )
	
	def __eventEditUnitNameBegin(self, argsList):
		pUnit = argsList
		popup = PyPopup.PyPopup(CvUtil.EventEditUnitName, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setUserData((pUnit.getID(),))
		popup.setBodyString(localText.getText("TXT_KEY_RENAME_UNIT", ()))
		popup.createEditBox(pUnit.getNameNoDesc())
		popup.launch()

	def __eventEditUnitNameApply(self, playerID, userData, popupReturn):	
		'Edit Unit Name Event'
		iUnitID = userData[0]
		unit = gc.getPlayer(playerID).getUnit(iUnitID)
		newName = popupReturn.getEditBoxString(0)
		if (len(newName) > 25):
			newName = newName[:25]			
		unit.setName(newName)

	def __eventWBAllPlotsPopupBegin(self, argsList):
		CvScreensInterface.getWorldBuilderScreen().allPlotsCB()
		return
	def __eventWBAllPlotsPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getButtonClicked() >= 0):
			CvScreensInterface.getWorldBuilderScreen().handleAllPlotsCB(popupReturn)
		return

	def __eventWBLandmarkPopupBegin(self, argsList):
		CvScreensInterface.getWorldBuilderScreen().setLandmarkCB("")
		#popup = PyPopup.PyPopup(CvUtil.EventWBLandmarkPopup, EventContextTypes.EVENTCONTEXT_ALL)
		#popup.createEditBox(localText.getText("TXT_KEY_WB_LANDMARK_START", ()))
		#popup.launch()
		return

	def __eventWBLandmarkPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getEditBoxString(0)):
			szLandmark = popupReturn.getEditBoxString(0)
			if (len(szLandmark)):
				CvScreensInterface.getWorldBuilderScreen().setLandmarkCB(szLandmark)
		return

	def __eventWBScriptPopupBegin(self, argsList):
		popup = PyPopup.PyPopup(CvUtil.EventWBScriptPopup, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setHeaderString(localText.getText("TXT_KEY_WB_SCRIPT", ()))
		popup.createEditBox(CvScreensInterface.getWorldBuilderScreen().getCurrentScript())
		popup.launch()
		return

	def __eventWBScriptPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getEditBoxString(0)):
			szScriptName = popupReturn.getEditBoxString(0)
			CvScreensInterface.getWorldBuilderScreen().setScriptCB(szScriptName)
		return

	def __eventWBStartYearPopupBegin(self, argsList):
		popup = PyPopup.PyPopup(CvUtil.EventWBStartYearPopup, EventContextTypes.EVENTCONTEXT_ALL)
		popup.createSpinBox(0, "", gc.getGame().getStartYear(), 1, 5000, -5000)
		popup.launch()
		return

	def __eventWBStartYearPopupApply(self, playerID, userData, popupReturn):
		iStartYear = popupReturn.getSpinnerWidgetValue(int(0))
		CvScreensInterface.getWorldBuilderScreen().setStartYearCB(iStartYear)
		return
		
# Rise of Mankind 2.7 Start
	def doInquisitorPersecution( self, pCity, pUnit ):
		revModEnabled = gc.getGame().isOption(GameOptionTypes.GAMEOPTION_REVOLUTION)
		pPlayer = gc.getPlayer( pCity.getOwner( ) )
		iPlayer = pPlayer.getID( )
		iGoldTotal = 0
		
		# gets a list of all religions in the city except state religion
		CyInterface().playGeneralSound("AS3D_UN_CHRIST_MISSIONARY_ACTIVATE")
		lCityReligions = [ ]
		for iReligionLoop in range(gc.getNumReligionInfos( )):
			if pCity.isHasReligion( iReligionLoop ):
				if pPlayer.getStateReligion( ) != iReligionLoop:
					lCityReligions.append( iReligionLoop )
					iHC = 0
					if pCity.isHolyCityByType(iReligionLoop):
						iHC = -50
				else:
					if pCity.isHolyCityByType(iReligionLoop):
						iHC = 10

		# Does Persecution succeed
		irandom = gc.getGame().getSorenRandNum(100,"")
		if irandom < 95 - ((len( lCityReligions ))*5) + iHC:
			
			# increases Anger for all AIs in City Religion List
			if len( lCityReligions ) > 0:
				lPlayers =PyGame().getCivPlayerList()
				for iAI_PlayersLoop in range( len(lPlayers) ):
					pSecondPlayer = lPlayers[iAI_PlayersLoop]
					iSecondPlayer = pSecondPlayer.getID( )
					for iAIAngerLoop in range( len( lCityReligions ) ):
						pReligion = gc.getReligionInfo( lCityReligions[iAIAngerLoop] )
						if pSecondPlayer.getStateReligion( ) == lCityReligions[iAIAngerLoop]:
							pSecondPlayer.AI_changeAttitude( pPlayer.getID( ), -1 )
							CyInterface().addMessage(iSecondPlayer,False,25,CyTranslator().getText("TXT_KEY_MESSAGE_INQUISITION_GLOBAL",(pCity.getName(),pReligion.getText())),"AS2D_PLAGUE",InterfaceMessageTypes.MESSAGE_TYPE_INFO,pUnit.getButton(),ColorTypes(7),pCity.getX(),pCity.getY(),True,True)
						else:
							if iSecondPlayer != iPlayer:
								lCities = PyPlayer( iSecondPlayer ).getCityList( )
								for iCityloop in range( pSecondPlayer.getNumCities() ):
									pPlayerCity = lCities[ iCityloop ]
									if pPlayerCity.isHolyCityByType(lCityReligions[iAIAngerLoop]):
										CyInterface().addMessage(iSecondPlayer,False,25,CyTranslator().getText("TXT_KEY_MESSAGE_INQUISITION_GLOBAL",(pCity.getName(),pReligion.getText())),"AS2D_PLAGUE",InterfaceMessageTypes.MESSAGE_TYPE_INFO,pUnit.getButton(),ColorTypes(7),pCity.getX(),pCity.getY(),True,True)

			# removes buildings
			for iBuildingLoop in range(gc.getNumBuildingInfos( )):
				if pCity.isHasBuilding( iBuildingLoop ):
					pBuilding = gc.getBuildingInfo( iBuildingLoop )
					#Shrines are excluded from consideration here
					iRequiredReligion = pBuilding.getPrereqReligion( )
					#Next line would also include shrines
					#iRequiredReligion = pBuilding.getReligionType( )
					for iReligionLoop in range(gc.getNumReligionInfos()):
						if iReligionLoop != pPlayer.getStateReligion( ):
							if iRequiredReligion == iReligionLoop:
								pCity.setNumRealBuilding ( iBuildingLoop,0 )

			# remove religions
			iStateReligion = gc.getPlayer( pUnit.getOwner( ) ).getStateReligion( )
			for iReligionLoop in range(gc.getNumReligionInfos()):
				if iReligionLoop != iStateReligion:
					if RevDCMOpt.isOC_RESPAWN_HOLY_CITIES():
						if pCity.isHolyCityByType(iReligionLoop):
							gc.getGame().clearHolyCity(iReligionLoop)
							pCity.setHasReligion(iReligionLoop, 0, 0, 0)
							minCitiesForInfluence = 2
							eBestPlayer = CvUtil.getPlayerWithMaxInfluenceForReligion(iReligionLoop, minCitiesForInfluence)
							if eBestPlayer >= 0:
								XpPlayer = gc.getPlayer(eBestPlayer)
								CvUtil.ReSpawnHolyCity(XpPlayer, iReligionLoop)
					if not pCity.isHolyCityByType(iReligionLoop):
						pCity.setHasReligion(iReligionLoop, 0, 0, 0)
						# Add gold to treasury to compensate for inquisition costs
# Rise of Mankind 2.7 adjustment how much gold you get from successful Inquisitor usage, gives now 20% of default setting
						iGoldTotal = iGoldTotal + 20 # default was 100

			# Persecution succeeds
			CyInterface().addMessage(CyGame().getActivePlayer(),False,25,CyTranslator().getText("TXT_KEY_MESSAGE_INQUISITION",(pCity.getName(),)),"AS2D_PLAGUE",InterfaceMessageTypes.MESSAGE_TYPE_INFO,pUnit.getButton(),ColorTypes(8),pCity.getX(),pCity.getY(),True,True)
			if revModEnabled:
				# Unhappiness from persecution
				pCity.changeHurryAngerTimer(pCity.flatHurryAngerLength())
				# RevolutionDCM - inquisitions
				pCity.changeRevolutionIndex(-100)
			else:
				pPlayer.changeGold(iGoldTotal)
		else:
			# Persecution fails
			CyInterface().addMessage(CyGame().getActivePlayer(),False,25,CyTranslator().getText("TXT_KEY_MESSAGE_INQUISITION_FAIL",(pCity.getName(),)),"AS2D_SABOTAGE",InterfaceMessageTypes.MESSAGE_TYPE_INFO,pUnit.getButton(),ColorTypes(7),pCity.getX(),pCity.getY(),True,True)
			if revModEnabled:
				# Unhappiness from attempted persecution
				pCity.changeRevolutionIndex(200)
				pCity.changeHurryAngerTimer(pCity.flatHurryAngerLength())
			else:
				pCity.changeHurryAngerTimer(pCity.flatHurryAngerLength())
				
		# Unit expended
		pUnit.kill( 0, -1 )
